#include "stdafx.h"
#include "header.h"

int CMainLoop::Main (void)
{
    // The main game loop.  Cycles through each faction's turn and
    // handles the display.
    
    bQuit = false;
    
    iWinner = -1;
    iStatWindow = -1;
    iStatPage = 0;
    
    mHP = gfxLoad("hp_bar", "elements");
    mArrows = gfxLoad("movement", "elements");
    mStatBG[0] = gfxLoad("stats_bg", "gui");
    mStatBG[1] = gfxLoad("stats_01", "gui");
    mStatBG[2] = gfxLoad("stats_02", "gui");
    mStatBG[3] = gfxLoad("stats_04", "gui");
    mStatBG[4] = gfxLoad("stat_row", "gui");
    colorReplace(mArrows, makecol(255, 128, 255), makecol(255, 0, 255));
    map.LoadTiles("001");
    
    iHeader = 0;
    bHeaderOn = true;
    iOffsetX = 0;
    iOffsetXX = 0;
    iOffsetY = 0;
    iOffsetYY = 0;
    iScrollSpeed = 4;
    
    playerTurn(-1);
    iFaction = 1;
    iFadeSpeed = 4;
    iFade = 0;
    
    // Find an active faction to start with.
    
    while(faction[iFaction].bActive == false)
    {
        iFaction ++;
        if(iFaction > giTotalFactions)
            iFaction = 0;
    };    
    
    // Play the first faction's theme song.
    
    music.Play(faction[iFaction].iMusic);
    
    while(!bQuit)
    {
        // Reset variables. If they need to be set to something else, the Player
        // and AI routines will do it themselves.
        
        bHeaderOn = true;
        
        // Get Commands from the Player or AI
        
        if(iFade == 0 && !battle.bActive && iStatWindow == -1)
        {
            int iReturn = 0;
            
            if(!bTurnHasStarted)
            {
                factionStart(iFaction);
                bTurnHasStarted = true;
            }
            else
                iReturn = playerTurn(iFaction);
            
            if(iReturn > 0)
            {
                // Bring up Stat Window
                iStatWindow = iReturn - 2;
            };
            
            // Faction has ended turn.
            
            if(iReturn == -1)
            {
                factionReset(iFaction);
                if(giTeams > 1)
                {
                    iFaction ++;
                    while(faction[iFaction].bActive == false)
                    {
                        iFaction ++;
                        if(iFaction > giTotalFactions)
                            iFaction = 0;
                    };
                }
                iFadeSpeed = 8;
                bTurnHasStarted = false;
                music.Play(faction[iFaction].iMusic);
            };
            
            // Function returned a call to quit.
            
            if(iReturn == -2)
            {
                bQuit = true;
                //iWinner = iFaction;
            };
        }
        
        // Perform the battle animation.
        
        else if(battle.bActive)
        {
            battle.Simple();
        };
        
        // Check on the teams and factions.
        
        FactionCheck_();
        
        // Check if a team has won.
        
        if(giTeams == 1)
        {
            bQuit = true;
            iWinner = iFaction;
        };
        
        // Perform the effects at the beginning of a faction's turn, as neccissary.
        
        if(iFadeSpeed != 0)
        {
            iFade += iFadeSpeed;
            if(iFade >= 256)
                iFadeSpeed = -4;
            if(iFade <= 0)
            {
                iFadeSpeed = 0;
                iFade = 0;
            };
        };
        
        // Handle Map Scrolling Goodness
		
		ScrollMap_();
		
		if(mouse_x > (SCREEN_W - giTileX) && iOffsetX < map.iSizeX - giTilesWide
			|| key[KEY_RIGHT] && iOffsetX < map.iSizeX - giTilesWide)
			MoveMap_(dRIGHT);
		if(mouse_x < giTileX && iOffsetX > 0
			|| key[KEY_LEFT] && iOffsetX > 0)
			MoveMap_(dLEFT);
		if(mouse_y > (SCREEN_H - giTileY) && iOffsetY < map.iSizeY - giTilesHigh
			|| key[KEY_DOWN] && iOffsetY < map.iSizeY - giTilesHigh)
			MoveMap_(dDOWN);
		if(mouse_y < giTileY && iOffsetY > 0
			|| key[KEY_UP] && iOffsetY > 0)
			MoveMap_(dUP);
        
        // Control Keys -- These need to be active even outside the player's turn.
        
        if(iFade != 0)
        {
            if(mouseClick(1) || mouseClick(2))
            {
                iFadeSpeed = 0;
                iFade = 0;
            };
        };
        if(keyCheck(KEY_ESC))
        {
            bQuit = true;
        };
		if(keyCheck(KEY_G))
			map.ToggleGrid();
        
        // Update the Mouse
        
        FindMouse_();
        
        // Run the display
        
        Display_();
    };    

    // clean everything up before going back.
    
    msg.ClearAll();
    unitKillAll();
    unitClear();
    guiKillAll();    
    destroy_bitmap(mHP);
    destroy_bitmap(mArrows);
    destroy_bitmap(mStatBG[0]);
    destroy_bitmap(mStatBG[1]);
    destroy_bitmap(mStatBG[2]);
    destroy_bitmap(mStatBG[3]);
    destroy_bitmap(mStatBG[4]);
    music.Play(0);
    music.Purge();
    
    return iWinner; // Done
};

int CMainLoop::FactionCheck_ (void)
{
    // Checks on all the factions and teams and sees if they are still alive.
                
    for(int i = 0; i < giTotalFactions; i++)
    {
        if(faction[i].bActive)
        {
            // Check to see if this faction has any units left.
            
            if(faction[i].iUnits < 1)
            {
                faction[i].bActive = false;
                giTeams--;
            };
    
            // Check to see if this faction has any flags left.
                        
            if(faction[i].iFlags == 0)
            {
                faction[i].bActive = false;
                giTeams--;
                
                // Delete all this faction's units.
                
                for(int j = 0; j < giMaxUnits; j++)
                {
                    if(unit[j].iFaction == i)
                        unit[j].bActive = false;
                };
            };
        };
    };
    
    return 0; // Done
};

int CMainLoop::SetupMap (void)
{
    // Sets the raw map data up to play a game.
    
    int iUnit = 0;
    
    giTeams = 0;
    
    // First, we must turn off all the factions.
    
    for(int i = 0; i < giTotalFactions + 1; i++)
    {
        faction[i].bActive = false;
        faction[i].iUnits = 0;
        faction[i].iFlags = -1;
        faction[i].iVIPs = -1;
    };
    
    // Now, parse the text data.
    
    parStrings(txt, "gameplay");
    
    // We need to transfer all the unit's starting positions
    // to actual units, and count how many flags and VIPs
    // there are.
    
    for(int x = 0; x < map.iSizeX; x++)
    {
        for(int y = 0; y < map.iSizeY; y++)
        {
            if(map.iFlag[x][y] != -1)
            {
                if(faction[map.iFlag[x][y]].iFlags == -1)
                    faction[map.iFlag[x][y]].iFlags = 1;
                else
                    faction[map.iFlag[x][y]].iFlags++;
            };
            if(map.iGeneric[x][y] != 0)
            {
                SetUnit_(iUnit, x, y);
                iUnit++;
            };
        };
    };
    
    return 0; // Done
};     

int CMainLoop::FindMouse_ (void)
{
	// Finds which tile the mouse is hovering over and
	// updates the variables accordingly.  This is done
	// by simply diving the mouse coordinates by the
	// size of an individual tile.

	iX = mouse_x / giTileX;
	iY = mouse_y / giTileY;
	iXPos = iX * giTileX;
	iYPos = iY * giTileY;
    giMTileX = iX + iOffsetX;
    giMTileY = iY + iOffsetY;

	return 0; // Success
};

int CMainLoop::MoveMap_ (int iDirection)
{
	// Moves the map in a given direction.
	
	static int iWaiterX;
	static int iWaiterY;
	
	if(iDirection == dRIGHT && iWaiterX > 0 || iDirection == dLEFT && iWaiterX > 0)
		iWaiterX--;
	if(iDirection == dUP && iWaiterY > 0 || iDirection == dDOWN && iWaiterY > 0)
		iWaiterY--;
	
	// Set up the scrolling
	
	if(iDirection == dRIGHT && iOffsetXX == 0 && iWaiterX == 0)
	{
        iOffsetXX = -1 * (giTileX / iScrollSpeed);
		iWaiterX = giTileX / (-1 * iOffsetXX);		
	};

	if(iDirection == dLEFT && iOffsetXX == 0 && iWaiterX == 0)
	{
		iOffsetXX = giTileX / iScrollSpeed;
		iWaiterX = giTileX / iOffsetXX;
	};	

	if(iDirection == dDOWN && iOffsetYY == 0 && iWaiterY == 0)
	{
		iOffsetYY = -1 * (giTileY / iScrollSpeed);
		iWaiterY = giTileY / (-1 * iOffsetYY);
	};	

	if(iDirection == dUP && iOffsetYY == 0 && iWaiterY == 0)
	{
		iOffsetYY = giTileY / iScrollSpeed;
		iWaiterY = giTileY / iOffsetYY;
	};
	
	return 0; // Success
};

int CMainLoop::ScrollMap_ (void)
{
	// Handles the actual pixel scrolling set up by MoveMap_
	
	if(iOffsetXX > 0)
		iOffsetXX += giTileX / iScrollSpeed;
	if(iOffsetXX < 0)
		iOffsetXX -= giTileX / iScrollSpeed;
	if(iOffsetYY > 0)
		iOffsetYY += giTileY / iScrollSpeed;
	if(iOffsetYY < 0)
		iOffsetYY -= giTileY / iScrollSpeed;
	
	if(iOffsetXX >= giTileX)
	{
		iOffsetX --;
		iOffsetXX = 0;
	};
	if(iOffsetXX <= -1 * giTileX)
	{
		iOffsetX ++;
		iOffsetXX = 0;
	};
	if(iOffsetYY >= giTileY)
	{
		iOffsetY --;
		iOffsetYY = 0;
	};
	if(iOffsetYY <= -1 * giTileY)
	{
		iOffsetY ++;
		iOffsetYY = 0;
	};
	
	return 0; // Success
};

int CMainLoop::SetUnit_ (int iUnit, int x, int y)
{
    int iClass = map.iGeneric[x][y];
    int iSetFaction = map.iFaction[x][y];
    
    // Set General
    
    unit[iUnit].tName = faction[iSetFaction].tName;// + " " + rank[iClass].tName;
    unit[iUnit].tDescription = faction[iSetFaction].tDescription;
    unit[iUnit].bActive = true;
    unit[iUnit].mImage = create_bitmap(rank[iClass].mUnit->w, rank[iClass].mUnit->h);
    unit[iUnit].iX = x * giTileX;
    unit[iUnit].iY = y * giTileY;
    unit[iUnit].iLocX = x;
    unit[iUnit].iLocY = y;
    unit[iUnit].iFaction = iSetFaction;
    unit[iUnit].iClass = iClass;
    
    // Set Level and EXP Data
    
    unit[iUnit].iLev = 1;
    unit[iUnit].iNext = rank[iClass].iNext;
    
    // Set Animation
    
    unit[iUnit].iSpeed = rank[iClass].iSpeed * rank[iClass].iSpeed;
    unit[iUnit].bMoving = false;
    
    // Set Stats
    
    unit[iUnit].iMaxHP = rank[iClass].iBHP;
    unit[iUnit].iHP = rank[iClass].iBHP;
    unit[iUnit].iSTR = rank[iClass].iBSTR;
    unit[iUnit].iDEX = rank[iClass].iBDEX;
    unit[iUnit].iMAG = rank[iClass].iBMAG;
    unit[iUnit].iCON = rank[iClass].iBCON;
    unit[iUnit].iRES = rank[iClass].iBRES;
    unit[iUnit].iSPD = rank[iClass].iBSPD;
    unit[iUnit].iAGL = rank[iClass].iBAGL;
    unit[iUnit].iLUK = rank[iClass].iBLUK;
    unit[iUnit].iBLD = rank[iClass].iBBLD;
    unit[iUnit].iMOV = rank[iClass].iBMOV;
    unit[iUnit].iMoves = rank[iClass].iBMOV;
    
    // Set Items
    
    unit[iUnit].iSlots = rank[iClass].iSlots;
    
    for (int i=0; i < giTotalSkills; i++)
    {
        unit[iUnit].iSkill[i] = rank[iClass].iSkill[i];
    }
    
    for (int i=0; i < rank[iClass].iSlots; i++)
    {
        unit[iUnit].tItem[i] = rank[iClass].tItem[i];
        unit[iUnit].bSlotActive[i] = rank[iClass].bSlotActive[i];
    }
    
    // Set Attack
    
    unit[iUnit].iNear = rank[iClass].iNear;
    unit[iUnit].iRange = rank[iClass].iRange;
    
    // Set AI
    
    unit[iUnit].bBeenAttacked = false;
    unit[iUnit].vPath.clear();
    
    // Set Faction
    
    if(!faction[iSetFaction].bActive)
    {
        faction[iSetFaction].bActive = true;
        giTeams++;
    };
    faction[iSetFaction].iUnits++;
    
    blit(rank[iClass].mUnit, unit[iUnit].mImage, 0, 0, 0, 0, rank[iClass].mUnit->w, rank[iClass].mUnit->h);
    colorSwap(unit[iUnit].mImage, 0, faction[iSetFaction].iColor);
    map.iGeneric[x][y] = 0;
    
    return 0; // Done
};                

int playMap (void)
{
    // Calls up the main loop class, then returns the value of the main loop.
    
    int iWinner;
    
    loop.SetupMap();
    iWinner = loop.Main();
      
    return iWinner; // Done
};
