#include "stdafx.h"
#include "header.h"

    // Integers for info windows.
    
    const int _iWinUnit = 4;
    const int _iTxtUnit1 = 8;
    const int _iTxtUnit2 = 9;
    const int _iWinTerrain = 3;
    const int _iTxtTerrain = 5;
    
class CPlayer
{
public:
    int ClearMenu (void);
    int Flush (void);
    int MenuItem (int iToReturn, string tToCall);
    int TakeTurn (int iFaction);
    
    bool bMenu;                     // Whether or not the menu is up.
private:    
    int DoBoxes_ (int iFaction);
    int DoMenu_ (void);
    int ManageInv_ (void);
    int UnitOrders_ (void);
    int UnitWait_ (void);

    bool bAttack;                   // Whether or not the player is choosing a target.
    
    int iAttackWindow;              // Whether or not the attack calculator display is up, and who it is displaying for.
    int iFunctionReturn;            // What value the main player function will return.
    int iInvHoldUses;               // How many uses the item that the player is holding currently has.
    int iInvUp;                     // Whether or not the player is managing a unit's inventory, and if so, whose.
    int iMenuItems;                 // How many options are currently on the menu.
    int iMouseAtOldX;               // The tile the mouse was hovering over before. (Used for path-finding.)
    int iMouseAtOldY;               // The tile the mouse was hovering over before. (Used for path-finding.)
    int iOldLocX;                   // If the player cancels an order, move the unit back to this x coordinate.
    int iOldLocY;                   // If the player cancels an order, move the unit back to this y coordinate.
    int iReturnValue[25];           // The return value of a given menu item.
    int iSelected;                  // Which unit is currently selected and being given orders to.
    int iStats;                     // Which unit's stats to display in the stat box.
    int iStatWindow;                // Who's Stat Window to View
    int iTarget;                    // Which unit the player wants to attack.
    
    string tInvHold;                // What item the player is holding. ("!NULL" for none.) Used for inventory management.
    string tName[25];               // The name of a given menu item.
} player;

int CPlayer::ClearMenu (void)
{
    // Clears the player's pop-up menu.
    
    iMenuItems = 0;
    win.Kill(5);
    
    for(int i = 0; i < 25; i++)
    {
        butKill(i + 10);
        txtKill(i + 10);
        image.Kill(i + 10);
    };
    
    // Clear the mouse.
    
    mouseClear();
    
    return 0; // Done
};

int CPlayer::Flush (void)
{
    iSelected = -1;
    iFunctionReturn = 0;
    iTarget = -1;
    bAttack = false;
    bMenu = false;
    iStatWindow = -1;
    iInvUp = -1;
    tInvHold = "!NULL";
    iInvHoldUses = 0;
    ClearMenu();
    
    return 0; // Done
};

int CPlayer::MenuItem (int iToReturn, string tToCall)
{
    // Adds an item to the menu.
    
    if(iMenuItems < 25)
    {
        iReturnValue[iMenuItems] = iToReturn;
        tName[iMenuItems] = tToCall;
        iMenuItems++;
    }
    else
        return -1; // Failure, the menu is already full.
    
    return 0; // Success
};

int CPlayer::TakeTurn (int iFaction)
{
    // Allows the player to move their characters. Returns -1 when turn
    // is ended by the player.
    
    // Display the info boxes as needed.
    
    iFunctionReturn = 0;    
    DoBoxes_(iFaction);
    iStatWindow = -1;
    
    if(!bMenu && iStatWindow == -1 && iInvUp == -1)
    {
        // Reset stuff related to the menu, etc. that we aren't using.
        
        iAttackWindow = -1;
        
        if(iSelected != -1 && unit[iSelected].iFaction == iFaction && !bAttack)
        {
            // The player is issuing movement orders, we want to track the
            // path that the player is trying to assign.
            
            if(iMouseAtOldX != giMTileX || iMouseAtOldY != giMTileY)
            {
                if(map.iRange[giMTileX][giMTileY] == 1 || map.iRange[giMTileX][giMTileY] == 2 || map.iRange[giMTileX][giMTileY] == 4)
                    unitAddPath(iSelected, giMTileX, giMTileY, false);
                //aiFindPath(iSelected, giMTileX, giMTileY, false);
                iMouseAtOldX = giMTileX;
                iMouseAtOldY = giMTileY;
            };
        };
        if(mouseClick(1))
        {
            // The user has clicked the left mouse button.
            
            if(iSelected == -1)
            {
                // No unit is selected, so we need to see if there is a unit in
                // this square to be selected.
                
                bool bActiveHere = false;       // Whether or not there's any active unit in this square;
                
                for(int i = 0; i < giMaxUnits; i++)
                {
                    if(unit[i].bActive == true && unit[i].iLocX == giMTileX && unit[i].iLocY == giMTileY && unit[i].iMoves != 0)
                    {
                        // There's a unit here; check his range.
                        
                        bActiveHere = true;
                        
                        if(unit[i].iMoves > 0)
                        {
                            unitAddPath(iSelected, giMTileX, giMTileY, false);
                            iMouseAtOldX = -1;
                            iMouseAtOldY = -1;
                            aiRange(i);
                            iSelected = i;
                            map.iGlowValue = 0;
                        };
                        
                        break;
                    }                        
                };

                if(!bActiveHere)
                {
                    // There is no unit here, the player wants to bring
                    // up the menu.
                    
                    //MenuItem(2, txt.Get("options"));
                    MenuItem(1, txt.Get("end"));
                    bMenu = true;
                };                
            }
            else if(iSelected != -1 && !bAttack)
            {
                // The player has a unit selected and is issuing movement orders.
                
                if(map.iRange[giMTileX][giMTileY] == 1 && unit[iSelected].iFaction == iFaction
                    || map.iRange[giMTileX][giMTileY] == 4 && unit[iSelected].iFaction == iFaction)
                {
                    // The unit can move here; tell it to do so, and store its old location
                    // in case the player changes their mind.

                    iOldLocX = unit[iSelected].iLocX;
                    iOldLocY = unit[iSelected].iLocY;                    
                    unit[iSelected].bMoving = true;
                    map.ClearRange();
                    
                    // Now that the unit has moved, display its options.
                    
                    UnitOrders_();
                    bMenu = true;                    
                }
                else
                {
                    // The player has clicked outside of the unit's range.
                    
                    //map.ClearRange();
                    //iSelected = -1;
                };                
            }
            else if(iSelected != -1 && bAttack && map.iRange[giMTileX][giMTileY] == 3)
            {
                // The player is choosing a target.
                
                bool bUnitHere = false;
                
                for(int i = 0; i < giMaxUnits; i++)
                {
                    if(unit[i].bActive && unit[i].iLocX == giMTileX && unit[i].iLocY == giMTileY && unit[i].iFaction != unit[iSelected].iFaction)
                    {
                        // There is a unit here that the player can attack
                        
                        bUnitHere = true;
                        iTarget = i;
                        battle.Calculate(iSelected, i);
                        UnitWait_();
                        i = giMaxUnits;
                    };                        
                };
                if(!bUnitHere)
                {
                    // There isn't a unit here, or it's not a space we can attack.
                    // Cancel the player's orders.
                    
                    
                };
            };
        }; //end if
        if(mouseClick(2))
        {
            // The player has clicked the right mouse button, and wants to cancel any
            // orders being issued, or they want to look at the stats of a given unit.
            
            if(!bAttack && iSelected == -1)
            {                
                for(int i = 0; i < giMaxUnits; i++)
                {
                    if(unit[i].bActive && unit[i].iLocX == giMTileX && unit[i].iLocY == giMTileY)
                    {
                        iStatWindow = i + 1;
                        i = giMaxUnits;
                    }
                    else
                        iStatWindow = -1;
                };
                //iStatWindow = iStats + 1;
                //iFunctionReturn = iStats + 1;
            }
            else if(!bAttack)
            {
                unit[iSelected].vPath.clear();
                map.ClearRange();
                iSelected = -1;
            }
            else if(bAttack)
            {
                iTarget = -1;
                map.ClearRange();
                UnitOrders_();
                bMenu = true;
                bAttack = false;
            };
        };    
    }
    else if(iInvUp != -1)
    {
        // The player is managing someone's inventory.
        ManageInv_();
    }
    else if(!unitMoving())
    {
        int iChoice = DoMenu_();
        
        if(iChoice < 0)
        {
            if(iSelected == -1)
            {
                // The player wants to close the menu.
                ClearMenu();
                bMenu = false;
            }
            else if(!bAttack)
            {
                // The player is canceling their movement orders.
                unit[iSelected].iLocX = iOldLocX;
                unit[iSelected].iLocY = iOldLocY;
                unit[iSelected].iX = unit[iSelected].iLocX * giTileX;
                unit[iSelected].iY = unit[iSelected].iLocY * giTileY;
                ClearMenu();                
                iSelected = -1;
            }
            else if(bAttack)
            {
                // The player is canceling the attack.
                bAttack = false;
                ClearMenu();
                map.ClearRange();
                UnitOrders_();                
                bMenu = true;
            };
        };
        switch(iChoice)
        {
            case 1:
                ClearMenu();
                return -1; // The player has ended their turn.
            case 2:
                ClearMenu();
                MenuItem(4, txt.Get("settings"));
                MenuItem(3, txt.Get("quit"));
                bMenu = true;
                break;
            case 3:
                ClearMenu();
                return -2; // The player wants to stop playing.
            case 4:
                // RESERVED for settings menu
                ClearMenu();
                break;
            case 5:
                // The player has instructed a unit to wait.
                UnitWait_();
                break;
            case 6:
                // The player wishes to attack.
                map.ClearRange();
                aiAttackRange(iSelected, unit[iSelected].iLocX, unit[iSelected].iLocY);
                bAttack = true;
                ClearMenu();
                bMenu = false;
                break;
            case 7:
                // The player wants to capture the flag in this square.
                {
                    int iFlagX = unit[iSelected].iLocX;
                    int iFlagY = unit[iSelected].iLocY;
                    faction[map.iFlag[iFlagX][iFlagY]].iFlags--;
                    if(map.iFlagType[iFlagX][iFlagY] == 0)
                        map.iFlag[iFlagX][iFlagY] = -1;
                    UnitWait_();
                }
                break;
            case 8:
                // The player is going to manage a character's inventory
                iInvUp = iSelected;
                ClearMenu();
                break;
        }; // end switch
    };
    
    //if(keyCheck(KEY_ENTER))
    //    return -1; // The player has ended their turn.
        
    return iFunctionReturn; // Done
};

int CPlayer::DoBoxes_ (int iFaction)
{
    // Displays the info boxes around the screen for the player
    
    // Unit Box
    
    for(int i = 0; i < giMaxUnits; i++)
    {
        if(unit[i].bActive && unit[i].iLocX == giMTileX && unit[i].iLocY == giMTileY)
        {
            iStats = i;
            i = giMaxUnits;
        }
        else
            iStats = -1;
    };
    
    if(iStats != -1 && !bMenu && !bAttack && iStatWindow == -1 && iInvUp == -1)
    {
        win.New(_iWinUnit, "window", 16, 288, 144, 4*16);
        txtNew(_iTxtUnit1, 88, 304, "", 2, -1, alignCENTER, 0);
        txtReset(_iTxtUnit1, unit[iStats].tName + "/n"
            + txt.Get("hp") + ": " + txtInt(unit[iStats].iHP) + "/" + txtInt(unit[iStats].iMaxHP));

        win.Move(_iWinUnit, loop.iHeader + 16, 288);
        txtMove(_iTxtUnit1,loop.iHeader + 88, 304);
    }
    else
    {
        win.Kill(4);
        txtKill(_iTxtUnit1);
        //txtKill(9);
    };
    
    // Terrain Box
    
    if(!bMenu && !bAttack && iStatWindow == -1 && iInvUp == -1)
    {
        win.New(_iWinTerrain, "window", 16, 368, 144, 96);
        txtNew(_iTxtTerrain, 88, 384, "", 2, -1, alignCENTER, 0);
        txtReset(_iTxtTerrain, terrain[terrainAt(giMTileX, giMTileY)].tName + "/n " + 
            txt.Get("def") + txtInt(terrain[terrainAt(giMTileX, giMTileY)].iDef) + "/n " +
            txt.Get("res") + txtInt(terrain[terrainAt(giMTileX, giMTileY)].iRes) + "/n " +
            txt.Get("avo") + txtInt(terrain[terrainAt(giMTileX, giMTileY)].iAvo));
        
        win.Move(_iWinTerrain, loop.iHeader + 16, 368);
        txtMove(_iTxtTerrain, loop.iHeader + 88, 384);
    }
    else
    {
        win.Kill(_iWinTerrain);
        txtKill(_iTxtTerrain);
    };

    // Item Management
    
    if(iInvUp != -1)
    {        
        string tItemText = "";          // Temporary string of text used by the display.
        
        loop.bHeaderOn = false;         // Turn off Faction Header
        
        // Get Item Info
        
        //for(int i=0; i < 5 && i < rank
        
        // Left Side
        
        guiEZ(16);
        txtNew(9, 1, 9, "", 5, -1, alignLEFT, 0);
        txtNew(10, 4, 13, "", 2, -1, alignLEFT, 0);
        txtReset(9, unit[iInvUp].tName);
        
        tItemText = "";
        for(int i = 0; i < unit[iInvUp].iSlots; i++)
        {
            if(unit[iInvUp].tItem[i] != "!NULL")
            {
                tItemText += item[itemGet(unit[iInvUp].tItem[i])].tName + "/n /n ";
                image.Kill(10 + i); // Make sure to clear any icon already in this slot.
                image.New(10 + i, item[itemGet(unit[iInvUp].tItem[i])].tIcon, "items", 2, 13 + i*2, 1);
            } // end if
            else
            {
                tItemText += "/n /n ";
                image.Kill(10 + i);
            } // end else
        } // end for
        txtReset(10, tItemText);
        //txtReset(10, "Short Sword/n /n Dagger/n /n Uberspear /n /n Herb/n /n Knight Grand Cross");
        
        win.New(6, "window", 1, 1, 14, 10);
        win.New(7, "window", 1, 12, 14, 17);
        
        // Create Buttons and Icons
        for(int i = 0; i < unit[iInvUp].iSlots; i++)
        {
            butNew(10 + i, 2, 13 + i*2, "plain_big", "", "", 0);
            butOpacity(10 + i, 128);
        } // end for
        
        guiEZ(1);
    }
    else
    {
        txtKill(9);
        win.Kill(6);
        win.Kill(7);
    }
    
    // Attack Info Box
    
    if(iStats != -1 && !bMenu && bAttack && unit[iStats].iFaction != iFaction && map.iRange[unit[iStats].iLocX][unit[iStats].iLocY] == 3)
    {
        int iDamageTo = unit[iSelected].iSTR - (unit[iStats].iCON + terrain[unitTerrain(iStats)].iDef);
        int iDamageFrom = unit[iStats].iSTR - (unit[iSelected].iCON + terrain[unitTerrain(iSelected)].iDef);
        int iDistanceFrom = aiHeuristic(unit[iSelected].iLocX, unit[iSelected].iLocY, unit[iStats].iLocX, unit[iStats].iLocY);
        
        loop.bHeaderOn = false;         // Turn off Faction Header
        
        if(iDamageTo < 0)
            iDamageTo = 0;
        if(iDamageFrom < 0)
            iDamageFrom = 0;
        
        if(iAttackWindow != iStats)
        {
            int iWindowOffset = 0;
            
            iAttackWindow = iStats;
            
            if(mouse_x < 320)
            {
                iWindowOffset = 51;
            };
            
            image.Kill(2);
            txtKill(3);
            txtKill(4);
            txtKill(5);            
            
            guiEZ(8);
            image.New(2, "prebattle_01", "gui", 1 + iWindowOffset, 1, 1);
            colorSwap(image.mImage[2], 0, faction[unit[iSelected].iFaction].iColor);
            colorSwap(image.mImage[2], 2, faction[unit[iStats].iFaction].iColor);
            txtNew(3, 14 + iWindowOffset, 3, "", 2, -1, alignCENTER, 0);
            txtNew(4, 7 + iWindowOffset, 9, "", 2, -1, alignCENTER, 0);
            txtNew(5, 21 + iWindowOffset, 9, "", 2, -1, alignCENTER, 0);
            //txtNew(6, 88, 64, "", 2, -1, alignCENTER, 0);
            guiEZ(1);
        };
        
        txtReset(3, unit[iSelected].tName + "/n/n/n" + txt.Get("hp") + "/n" + txt.Get("mp") + "/n"
            + txt.Get("mt") + "/n" + txt.Get("hit") + "/n" + txt.Get("crt") + "/n" + "/n" + unit[iStats].tName + "/n" + "");
        txtReset(4, txtInt(unit[iSelected].iHP) + "/n" + txtInt(unit[iSelected].iMP)
            + "/n " + txtInt(iDamageTo) + "/n" + txtInt(unitHit(iSelected) - unitAvoid(iStats)) + "%/n" + "0%");
        if(iDistanceFrom >= unit[iStats].iNear && iDistanceFrom <= unit[iStats].iNear + unit[iStats].iRange && !rank[unit[iStats].iClass].bNoCounter)
        {
            txtReset(5, txtInt(unit[iStats].iHP) + "/n" + txtInt(unit[iStats].iMP)
                + "/n " + txtInt(iDamageFrom) + "/n" + txtInt(unitHit(iStats) - unitAvoid(iSelected)) + "%/n" + "0%");
        }
        else
            txtReset(5, txtInt(unit[iStats].iHP) + "/n" + txtInt(unit[iStats].iMP));
    }
    else
    {
        iAttackWindow = -1;
        image.Kill(2);
        txtKill(3);
        txtKill(4);
    }; 
    
    // Stat Window Handling
    
    if(iStatWindow != -1)
    {        
        iFunctionReturn = iStatWindow + 1;
    }

    return 0; // Done
};

int CPlayer::DoMenu_ (void)
{
    // Displays the player's pop-up menu, and returns a value
    // if the player has made a selection. Otherwise, returns
    // a 0. If there are no items on the menu, then it will
    // return a -2. If the user wants to close the menu, it
    // will return a -1.
    
    int iAtX = mouse_x - 72;
    int iAtY = mouse_y + 16; //- 32 - iMenuItems*16;
    
    if(iAtY < 32)
        iAtY = 32;
    if(iAtY > 416 - iMenuItems*16)
        iAtY = 416 - iMenuItems*16;
    if(iAtX < 32)
        iAtX = 32;
    if(iAtX > 464)
        iAtX = 464;    
    
    if(iMenuItems == 0)
        return -2;  // Failure, there are no menu items.
    
    win.New(5, "window", iAtX, iAtY, 144, 32 + iMenuItems*16);
    
    for(int i = 0; i < iMenuItems; i++)
    {
        butNew(i + 10, iAtX + 16, iAtY + (i*16) + 16, "plain_small", "", "", 0);
        txtNew(i + 10, iAtX + 36, (iAtY + 16 + i*16)+2, tName[i], 2, -1, alignLEFT, 0);
        butOpacity(i + 10, 127);
        
        // Get Command Icons

        if(iReturnValue[i] == 1 && !image.bActive[i + 10])
            image.New(i + 10, "icon_end", "elements", iAtX + 16, iAtY + (i*16) + 16, 1);        
        if(iReturnValue[i] == 5 && !image.bActive[i + 10])
            image.New(i + 10, "icon_wait", "elements", iAtX + 16, iAtY + (i*16) + 16, 1);
        if(iReturnValue[i] == 6 && !image.bActive[i + 10])
            image.New(i + 10, "icon_attack", "elements", iAtX + 16, iAtY + (i*16) + 16, 1);
        if(iReturnValue[i] == 7 && !image.bActive[i + 10])
            image.New(i + 10, "icon_capture", "elements", iAtX + 16, iAtY + (i*16) + 16, 1);
        if(iReturnValue[i] == 8 && !image.bActive[i + 10])
            image.New(i + 10, "icon_inventory", "elements", iAtX + 16, iAtY + (i*16) + 16, 1);        
        
        if(butCheck(i + 10))
        {
            mouseClear();
            return iReturnValue[i];
        };
    };
    if(mouseClick(2))
        return -1;  // Done, the player wants to close the menu.
    
    return 0; // Done, the player has not made a selection.    
};

int CPlayer::ManageInv_ (void)
{
    // Displays a unit's inventory and checks what the player is
    // doing with it.
    
    // Go through all the buttons and check them.
    //
    // TODO: Add restrictions to keep player from altering locked inventories/slots/items/etc.
    
    for(int i = 0; i < unit[iInvUp].iSlots; i++)
    {
        if(butCheck(10 + i))
        {
            // Swap the item in the player's "hand" with the item in the
            // selected inventory slot.
            
            string tSwapItem; // Temp string used to swap out data.
            
            tSwapItem = tInvHold;
            tInvHold = unit[iInvUp].tItem[i];
            unit[iInvUp].tItem[i] = tSwapItem;
        } // end if
    } // end for

    
    
    // Check for exit
    
    if(mouseClick(2) && tInvHold == "!NULL")
    {
        // Delete Buttons
        for(int i = 0; i < unit[iInvUp].iSlots; i++)
        {
            butKill(10 + i);
            image.Kill(10 + i);
        } // end for
        
        iInvUp = -1;
    } // end if
    
    return 0;
}

int CPlayer::UnitOrders_ (void)
{
    // Checks which orders can be given to a unit, then adds them
    // to the menu.
    
    int iEndX = unit[iSelected].iLocX;      // The unit's X desitnation
    int iEndY = unit[iSelected].iLocY;      // The unit's Y destination
    
    if(unit[iSelected].vPath.size() > 0)
    {
        iEndX = unit[iSelected].vPath[unit[iSelected].vPath.size() - 1].iX;
        iEndY = unit[iSelected].vPath[unit[iSelected].vPath.size() - 1].iY;
    };
    
    if(unitCanAttack(iSelected, iEndX, iEndY))
    {
        // Some units cannot attack and move on the same turn, so we need to see
        // what the status of this unit is, even if it is otherwise able to
        // attack normally.
        
        if(!rank[unit[iSelected].iClass].bNoRunAndGun)
            MenuItem(6, txt.Get("attack"));
        else if(unit[iSelected].vPath.size() < 2)
            MenuItem(6, txt.Get("attack"));
    };
    
    if(map.iFlag[iEndX][iEndY] != -1 && map.iFlag[iEndX][iEndY] != unit[iSelected].iFaction)
        MenuItem(7, txt.Get("capture"));
    MenuItem(8, txt.Get("inventory"));
    MenuItem(5, txt.Get("wait"));
    MenuItem(-1, txt.Get("cancel"));
    
    return 0; // Done
};

int CPlayer::UnitWait_ (void)
{
    // Instructs the given unit to wait.
    
    map.ClearRange();
    colorGray(unit[iSelected].mImage);
    unit[iSelected].iMoves = 0;
    iSelected = -1;
    iTarget = -1;
    ClearMenu();
    bMenu = false;
    bAttack = false;
    
    return 0; // Success
};

bool playerBusy (void)
{
    // Checks if the player has the menu up, etc.
    
    if(player.bMenu == true)
        return true; // The player has the menu up.
    else
        return false; // The player is issuing normal commands.
};

int playerTurn (int iFaction)
{
    // Simply calls and returns the calue of player.TakeTurn.
    // If -1 is passed, all the data is flushed, so that the next
    // faction can take their turn.
    
    if(iFaction == -1)
        return player.Flush();
    else
        return player.TakeTurn(iFaction); // Done
};
