#include "stdafx.h"
#include "header.h"

// class CMap's functions.

int CMap::Clear (void)
{
	// Clears all the map data so that "artifacts" from old maps
	// won't show up when a new map is loaded.

    ClearFlags_();
	ClearHeader_();
    ClearRange();
	ClearTerrain_();
    ClearUnits_();

	return 0;	// Success
};

int CMap::ClearFlags_ (void)
{
    // Clears flag data.
    
    for(int x = 0; x < maxWIDTH; x++)
    {
        for(int y = 0; y < maxHEIGHT; y++)
        {
            tFlagCommand = "";
            iFlag[x][y] = -1;
            iFlagType[x][y] = 0;
            iFlagOwner[x][y] = -1;
        }; // end y
    }; // end x
    
    return 0; // Done
};

int CMap::ClearHeader_ (void)
{
	// Clears the header data.

	tName = "";
	tAuthor = "";
	tDescription = "";
	tVersion = "";
	iSizeX = giTilesWide;
	iSizeY = giTilesHigh;
	iTileSet = 0;

	return 0;	// Done
};

int CMap::ClearRange (void)
{
    // Clears the movement range colors from the map.
    
    for(int x = 0; x < maxWIDTH; x++)
    {
        for(int y = 0; y < maxHEIGHT; y++)
        {
            iRange[x][y] = 0;
        }; // end y
    }; // end x
    
    return 0; // Done
};

int CMap::ClearTerrain_ (void)
{
	// Clears the terrain data

	for(int layer = 0; layer < 4; layer++)
	{
		for(int x = 0; x < maxWIDTH; x++)
		{
			for(int y = 0; y < maxHEIGHT; y++)
			{
                iTile[layer][x][y] = 0;
				if(layer == 0)
					iTerrain[layer][x][y] = 1; // If it's the base layer, set the default tile.
				else
					iTerrain[layer][x][y] = 0; // Otherwise, set an "empty" null tile.
			}; // end x for
		}; // end y for
	}; // end layer for
    
	// Now, destory and clear the bitmap to avoid memory leaks.

	destroy_bitmap(mTiles);
	mTiles = NULL;

	return 0;	// Done
};

int CMap::ClearUnits_ (void)
{
    // Clears unit data.
    
    giTotalUnits = 0;
    
    // Clear the generic unit data

    for(int x = 0; x < maxWIDTH; x++)
	{
		for(int y = 0; y < maxHEIGHT; y++)
		{            
			iGeneric[x][y] = 0;
            bActiveVIP[x][y] = false;
            bPassiveVIP[x][y] = false;
		}; // end x for
    }; // end y for 
    
    return 0; // Done
};

int CMap::LoadTiles (string tFileName)
{
	// This function will load the map tiles to be used.

	mTiles = gfxLoad(tFileName, "tiles");
	colorReplace(mTiles, makecol(255, 128, 255), makecol(255, 0, 255));

	return 0; // Success
};

int CMap::Terrain (int iX, int iY)
{
    // Returns the ID of the top-most terrain at a given map cordinate.
    
    int iTerrain = 0;
    
    for(int l = 3; l > -1; l--)
    {
        if(map.iTerrain[l][iX][iY] != 0)
        {
            iTerrain = map.iTerrain[l][iX][iY];
            l = -1;
        };
    };    
    
    return iTerrain;
};    

int CMap::ToggleGrid (void)
{
	// Toggles the grid display on or off.
	
	if(bGrid)
		bGrid = false;
	else
		bGrid = true;
	
	return 0; // Success
};
