#include "stdafx.h"
#include "header.h"

int CMusic::Load (int iSlot, string tFile, int iStart, int iLoop)
{
    // Loads MIDI file data for the engine to play later.
    
    if(iSlot < 1 || iSlot > sys_iMaxBGMs)
        return -1; // Error! Invalid slot number.
        
    tBGM[iSlot] = tFile;
    iBGMStart[iSlot] = iStart;
    iBGMLoop[iSlot] = iLoop;
    
    return 0; // Success
};

int CMusic::Play (int iBGM)
{
    // Plays the selected BGM.
    
    if(iBGM < 0 || iBGM > sys_iMaxBGMs)
        return -1; // Error! Invalid slot number.
    
    if(iNowPlaying != iBGM)
    {
        stop_midi();
        destroy_midi(vBGM);     // Clears whatever song may be playing.
        vBGM = NULL;
    }
    else
        return 0; // Done. We're already playing the requested song.
    
    iNowPlaying = iBGM;
    
    if(tBGM[iBGM] == "")
        return -2; // Error! This slot is empty.
    
    // Unless we're passed a 0 (meaning just stop whatever song is
    // playing) we load and play the given selection.
    
    if(iBGM != 0)
    {
        //string tLoad = "shared/rulesets/classic/sound/bgm/blue.mid";
        string tLoad = sys_tPath + sys_tMusicPath + tBGM[iBGM] + ".mid";    
        
        if(!exists(tLoad.c_str()))
        {
            allegro_message("Music Error");
            exit(-1);
        };
    
        vBGM = load_midi(tLoad.c_str());
        if(sys_bMusicOn)
            play_looped_midi(vBGM, iBGMStart[iBGM], iBGMLoop[iBGM]);
    };
    
    return 0; // Success
};

int CMusic::Purge (void)
{
    // Clears any and all music data from the system.
    
    stop_midi();
    destroy_midi(vBGM);     // Clears whatever song may be playing, and frees vBGM.
    
    for(int i = 0; i < sys_iMaxBGMs; i++)
    {
        tTitle[i] = "";
        tBGM[i] = "";
        iBGMStart[i] = 0;
        iBGMLoop[i] = 0;
        iBGMEnd[i] = -1;
        iNowPlaying = 0;
    };
    
    return 0; // Done
};

int CMusic::Replay (void)
{
    // Replays the last song loaded *FROM THE BEGINNING*.
    
    if(sys_bMusicOn && iNowPlaying != 0)
        play_looped_midi(vBGM, iBGMStart[iNowPlaying], iBGMLoop[iNowPlaying]);

    return 0; // Success
};

int CMusic::Stop (void)
{
    // Stops whatever music is playing.
    
    stop_midi();
    //iNowPlaying = 0;
    //destroy_midi(vBGM);     // Clears whatever song may be playing.
    //vBGM = NULL;
    
    return 0; // Done
};

int musicLoad (string tFile)
{
	// A routine for parsing music data from a file.
	
	FILE * pFile;					// A pointer to the file.
	string tInput;					// Input from the parser.
	
    // Clear any BGM data in memory.
    
    music.Purge();
    
	// First, we need to make sure the file is actually there. If it's not,
    // well, then we skip it and you're just not getting any music. :)
	
	if(!exists(tFile.c_str()))
	{
		return -1; // Done. There is no music data to parse.
	}; //end if
    
    // Now open the file.
    
	pFile = fopen(tFile.c_str(), "rt");
	
	// ...And begin parsing the data. We keep going until we hit the end of the file.
	
	while(!feof(pFile))
	{
		// Get the next command.
		
		tInput = parNext(pFile);
        
        // BGM
        
		if(tInput == "BGM")
		{
			int iSlot = 0;				// The slot number this terrain will take.
            
			// Get the BGM's slot number
			
			tInput = parNext(pFile);
			iSlot = atoi(tInput.c_str());
			
			// Then get the music's title.
            
            music.tTitle[iSlot] = parNext(pFile);
            
            // Now start processing data
            
            tInput = parNext(pFile);
            
            while(tInput != "}" && !feof(pFile))
            {
                // file name
                
                if (tInput == "file")
                {
                    parNext(pFile);
                    music.tBGM[iSlot] = parNext(pFile);
                };
                
                tInput = parNext(pFile);
            };
        };
        
		// If we ever hit a # comment line, then we want to skip the
		// entire thing. We'll have to handle that locally, so we
		// create a new char variable to deal with it.
		
		if(tInput == "#")
		{
			char cSingle;
            
            cSingle = (char)fgetc(pFile);
			
			while(cSingle != '\n')
			{
				cSingle = (char)fgetc(pFile);
			}; // end while
		}; // end if COMMENTS        
    };
    
    return 0; // Success
};
